#!/usr/bin/env python

import os
import os.path
import re
import struct
import sys

rec_tagline = re.compile("^traceplay gold ([-\\w]+) ([-\\w]+) (\\d+) (\\d+) content_size=(\\d+)$")

app = sys.argv.pop(0)
in_fn = sys.argv.pop(0)
out_fn = sys.argv.pop(0)

in_f = file(in_fn, "rb")

tagline = in_f.readline()
tagline.strip()

match = rec_tagline.match(tagline)
if not match:
    print >>sys.stderr, "tagline does not match format:"
    print >>sys.stderr, "    " + tagline
    sys.exit(1)

if match.group(1) != 'content':
    print >>sys.stderr, "File contains '%s' not 'content'" % match.group(1)
    sys.exit(1)

if not (match.group(2) in ['rgb', 'rgb-vinvert', 'bmp']):
    print >>sys.stderr, "File is '%s' not 'rgb' or 'rgb-vinvert'" % match.group(2)
    sys.exit(1)

if match.group(2) == 'bmp':
    is_bmp = True
    header_size = 54
else:
    header_size = 0
    is_bmp = False

if match.group(2) == 'rgb':
    heightsign = -1
else:
    heightsign = 1

try:
    width = int(match.group(3))
except:
    print >>sys.stderr, "Can't convert width '%s' to integer" % match.group(3)
    print >>sys.stderr, ""
    raise

try:
    height = int(match.group(4))
except:
    print >>sys.stderr, "Can't convert height '%s' to integer" % match.group(4)
    print >>sys.stderr, ""
    raise

try:
    content_size = int(match.group(5))
except:
    print >>sys.stderr, "Can't convert content_size '%s' to integer" % match.group(5)
    print >>sys.stderr, ""
    raise

if content_size != ((width * height * 3) + header_size):
    print >>sys.stderr, "content_size %d + header_size %d v.s. width %d x height %d mismatch" % (content_size, header_size, width, height)
    sys.exit(1)

if (not is_bmp) and ((width * 3) % 4):
    print >>sys.stderr, "width %d unsupported by conversion script; (bmp requires pitch be a multiple of 4, but width*3 isn't)" % width
    sys.exit(1)

content = in_f.read(content_size)

in_f.close()

if not is_bmp:
    bmp_header = struct.pack(
        '<ccIHHIIiiHHIIIIII',
        'B',                 # magic
        'M',                 # magic
        54 + len(content),   # file size
        0,                   # reserved
        0,                   # reserved
        54,                  # offset to bitmap data
        40,                  # size of rest of header
        width,               # image width
        heightsign * height, # image height (negative means top->bottom raster order)
        1,                   # number of color planes; must be 1
        24,                  # bits per pixel
        0,                   # compression; = none
        len(content),        # image data size
        96,                  # horizontal dpi
        96,                  # vertical dpi
        0,                   # number of palette entries; 0 defaults to 2**n
        0                    # number of important colors; 0 defaults to all
    )
    if len(bmp_header) != 54:
        print >>sys.stderr, "Internal error: len(bmp_header) != 54 (it's %d)" % len(bmp_header)
        sys.exit(1)

out_f = file(out_fn, "wb")
if not is_bmp:
    out_f.write(bmp_header)
out_f.write(content)
out_f.close()

