#ifndef _TRACEPLAY_TRACEPLAY_H
#define _TRACEPLAY_TRACEPLAY_H

#include <stdlib.h>
#include <vdpau/vdpau.h>

#ifdef __cplusplus
extern "C" {
#endif

// Data structures

#define TRACEPLAY_CMD_DECODE  0
#define TRACEPLAY_CMD_DISPLAY 1
#define TRACEPLAY_CMD_END     2

typedef struct {
    int                  vid_surface_index;
    VdpPictureInfo *     picture_info;
    uint32_t             bitstream_buffer_count;
    VdpBitstreamBuffer * bitstream_buffers;
} Traceplay_CmdDecode;

typedef struct {
    int vid_surface_index;
} Traceplay_CmdDisplay;

typedef struct {
    int    cmd;
    void * data;
} Traceplay_Cmds;

typedef struct {
    uint32_t          vid_width;
    uint32_t          vid_height;
    uint32_t          vid_surface_count;
    uint32_t          out_width;
    uint32_t          out_height;
    VdpDecoderProfile decoder_profile;
    uint32_t          decoder_max_references;
} Traceplay_Stream_Conf;

typedef enum {
    TRACEPLAY_PLAY_TYPE_OUTPUT,
    TRACEPLAY_PLAY_TYPE_VIDEO,
    TRACEPLAY_PLAY_TYPE_CUSTOM_MIXER,
} Traceplay_Play_Type;

struct Traceplay_Ctx;

// Callbacks

typedef void * (*Traceplay_Video_Surface_Post_Create)(
    struct Traceplay_Ctx * ctx,
    VdpVideoSurface        surface
);

typedef void * (*Traceplay_Output_Surface_Post_Create)(
    struct Traceplay_Ctx * ctx,
    VdpOutputSurface       surface
);

typedef void (*Traceplay_Video_Surface_Pre_Destroy)(
    struct Traceplay_Ctx * ctx,
    VdpVideoSurface        surface,
    void *                 surface_context
);

typedef void (*Traceplay_Output_Surface_Pre_Destroy)(
    struct Traceplay_Ctx * ctx,
    VdpOutputSurface       surface,
    void *                 surface_context
);

typedef void (*Traceplay_Display_Video_Surface)(
    struct Traceplay_Ctx * ctx,
    uint32_t               frame_number,
    VdpVideoSurface        surface,
    void *                 surface_context
);

typedef void (*Traceplay_Wait_For_Output_Surface_Idle)(
    struct Traceplay_Ctx * ctx,
    VdpOutputSurface       surface,
    void *                 surface_context
);

typedef void (*Traceplay_Display_Output_Surface)(
    struct Traceplay_Ctx * ctx,
    uint32_t               frame_number,
    VdpOutputSurface       surface,
    void *                 surface_context
);

typedef void (*Traceplay_Process_Surface)(
    struct Traceplay_Ctx * ctx,
    uint32_t               frame_number,
    VdpVideoSurface        vid_surface,
    void *                 vid_surface_context,
    VdpOutputSurface       out_surface,
    void *                 out_surface_context
);

typedef struct {
    Traceplay_Video_Surface_Post_Create    video_surface_post_create;
    Traceplay_Output_Surface_Post_Create   output_surface_post_create;
    Traceplay_Video_Surface_Pre_Destroy    video_surface_pre_destroy;
    Traceplay_Output_Surface_Pre_Destroy   output_surface_pre_destroy;
    Traceplay_Display_Video_Surface        display_video_surface;
    Traceplay_Wait_For_Output_Surface_Idle wait_for_output_surface_idle;
    Traceplay_Display_Output_Surface       display_output_surface;
    Traceplay_Process_Surface              process_surface;
} Traceplay_Callbacks;

// API

extern struct Traceplay_Ctx * traceplay_init(
    VdpDevice               vdp_device,
    VdpGetProcAddress *     vdp_get_proc_address,
    Traceplay_Play_Type     play_type,
    Traceplay_Callbacks *   callbacks,
    Traceplay_Stream_Conf * stream_conf
);

extern void traceplay_fini(
    struct Traceplay_Ctx * ctx
);

extern void traceplay_get_vid_surfaces(
    struct Traceplay_Ctx * ctx,
    uint32_t *             surface_count,
    VdpVideoSurface * *    surfaces
);

extern void traceplay_get_out_surfaces(
    struct Traceplay_Ctx * ctx,
    uint32_t *             surface_count,
    VdpOutputSurface * *   surfaces
);

extern void traceplay_play(
    struct Traceplay_Ctx * ctx,
    Traceplay_Cmds *       cmds,
    int                    repeat
);

#ifdef __cplusplus
}
#endif

#endif

